/*
* Copyright by Cadence Design Systems 2014. All rights reserved
*/
#ifndef __PSpiceCMIApiDefs_H__
#define __PSpiceCMIApiDefs_H__

#include "PSpiceCommonAPIDefs.h"

class qi_def;

/*! \page Introduction
 * This file lists 3 types of functions:
 *\n 1. Templates for functions exposed by PSpice Engine - their names follow the convention pPSpice*_t
 *\n    These are helper functions, and may be used by the Model Dll.
 *\n    Their function pointers are set into the Model Dll by the API pspiceSetFunctionList
 *\n
 *\n 2. Templates for API's that need to be exposed by the Model Dll - their names follow the convention p*_t
 *\n    Their pointers will be set by the Model Dll into PSpice Engine using the descSet* Functions.
 *\n
 *\n 3. Templates for descSet functions which may be used by the model dll to set function pointers into the PSpice Engine.
 *\n    Their names follow the convention descSet*
 *\n    Dll must export a function " void PSpiceInstallFunction();  "
 *\n    Inside this function, function pointers for each model install function must be set 
 *\n    All function pointers are set by this function.
 */

/*! \section 1.1
 * Templates for Functions exposed by PSpice Engine - these are helper functions, and may be used by the Model Dll
 * Their function pointers are set into the Model Dll by the API pspiceSetFunctionList
 */

/**
 * @param	pTitle	Text Title for each section in dat file
 * @return	None
 */
typedef void (*pPSpiceSetProbeTitle_t)(const char* pTitle);

/**
 * @param[in]	pNode1	Name of Node1
 * @param[in]	pNode2	Name of Node2
 * @return		Voltage calculated between Node1 and Node2
 */
typedef double (*pPSpiceGetVoltageNodes_t)(const char* pNode1, const char* pNode2);

/**
 * @param[in]	pNode1	Name of Node1
 * @param[in]	pNode2	Name of Node2
 * @return		Imaginary part of Voltage calculated between Node1 and Node2 (valid for ac analysis only)
 */
typedef double (*pPSpiceGetVoltageNodesI_t)(const char* pNode1, const char* pNode2);

/**
 * @param	None
 * @return	State index currently being used by engine (valid for transient analysis)
 */
typedef int (*pPSpiceGetCurrentStateIndex_t)(void);

/**
 * @param	None
 * @return	Get current Time Step value (valid for transient analysis)
 */
typedef double (*pPSpiceGetDelta_t)(void);

/**
 * @param[in]	pTimeStepIndex	Previous Time Step Index (valid values are 1|2|3
 * @return		Get the respective Time Step value (valid for transient analysis)
 */
typedef double (*pPSpiceGetDeltaPrevious_t)(int pTimeStepIndex);

/**
 * @param[in]	pValue1	First voltage value
 * @param[in]	pValue2	Second voltage value
 * @return		Maximum Voltage tolerance for the voltage differential (depending on RELTOL and VNTOL)
 */
typedef double (*pPSpiceVoltageTolerance_t)(double pValue1, double pValue2);

/**
 * @param	None
 * @return	Get current Frequency value (valid for ac analysis)
 */
typedef double (*pPSpiceGetFrequency_t)(void);

/**
 * @param[out]	pConductance	Equivalent conductance calculated based on integrated charges and currents
 * @param[out]	pCurrent		Equivalent current calculated based on integrated charges and currents
 * @param[in]	pCapacitance	Capacitance of branch
 * @param[in]	pSV0			Current state vector
 * @param[in]	pSV1			Previous first state vector
 * @param[in]	pSV2			Previous second state vector
 * @param[in]	pInitFlag		Init Flag
 * @return		None
 */
typedef void (*pPSpiceIntegrate_t)(double& pConductance, double& pCurrent, double pCapacitance, qi_def& pSV0, qi_def& pSV1, qi_def& pSV2, int pInitFlag);

/**
 * @param[in]	pQI0	Current state vector
 * @param[in]	pQI1	Previous first state vector
 * @param[in]	pQI2	Previous second state vector
 * @param[in]	pQI3	Previous third state vector
 * @param[in]	pDevice	Device Pointer
 * @return		Truncation error in terms of minimum time step recommended by the device
 */
typedef double (*pPSpiceCurrentTErr_t)(const qi_def* pQI0,  const qi_def* pQI1,  const qi_def* pQI2,  const qi_def* pQI3,	void* pDevice);

/**
* @param[in]
*/
typedef void (*pPSpiceUpdateStateVector_t)(char**, char*, int);

typedef void (*pPSpiceSetSimulationTemperature_t)(double);

/**
 * @param[in]	var		Parameter on which internal node addition is dependent
 * @param[out]	inode	Internal node - internal node created if var != 0
 * @param[in]	xnode	Internal node shorted to existing node xnode if var == 0
 */
typedef void (*pPSpiceAddInternalNode_t)(double var, char*& inode, const char* xnode);

/**
 * @param[in]	var		Parameter on which internal node addition is dependent
 * @param[out]	inode	Internal node - internal node created if var != 0
 * @param[in]	xnode	Internal node shorted to existing node xnode if var == 0
 * @param[in]	InternalNodeName	Default Name for the new node
 */
typedef void (*pPSpiceAddInternalNodeByName_t)(double var, char*& inode, const char* xnode, const char* InternalNodeName);

/**
 * @param[out]	pMatrixValueItem	Pointer to matrix value item - generated for the input node names
 * @param[in]	pNode1				Name of Node1 
 * @param[in]	pNode2				Name of Node2 
 * @return		None
 */
typedef void (*pPSpiceGetMatrixPtr_t)(double** pMatrixValueItem, const char* pNode1, const char* pNode2);

/**
 * @param[out]	pMatrixValueItem	Pointer to Matrix RHS - generated for the input node name
 * @param[in]	pNode1				Name of Node1 
 * @return		None
 */
typedef void (*pPSpiceGetRHSPtr_t)(double** pRHSPtr, const char* pNode);

/**
 * @param[in]	pMatrixValueItem	Pointer to real matrix value item
 * @param[in]	pValue				Real value to be written in matrix
 * @return		None
 */
typedef void (*pPSpiceApplyValueItem_t)(double* pMatrixValueItem, double pValue);

/**
 * @param[in]	pMatrixValueItem	Pointer to complex matrix value item
 * @param[in]	pRealValue			Real value to be written in matrix
 * @param[in]	pImagValue			Imaginary value to be written in matrix
 * @return		None
 */
typedef void (*pPSpiceApplyValueItemComplex_t)(double* pMatrixValueItem, double pRealValue, double pImagValue);


/**
 * @param[in]	pMatrixValueItem	Pointer to matrix value item
 * @param[in]	pValue				Real value to be added to an existing matrix value item
 * @return		None
 */
typedef void (*pPSpiceAdjustValueItem_t)(double* pMatrixValueItem, double pValue);


typedef void (*pPSpiceSetPWLDataDbl_t)(void* pRef, double* pX, double* pY, int pSize);

typedef void (*pPSpiceSetPWLDataStr_t)(void* pRef, const char* pStr, int pSize);

typedef void *PrimitivePtr;

/**
 * This class defines basic PSpice Device Model which will be used to transmit data to and from CMI Models.
 */
class PSpiceDeviceModel {
public:
	char* mModelID;
	void* mData; 
};

/**
 * This class defines basic PSpice Device Instance which will be used to transmit data to and from CMI Models.
 */
class PSpiceDeviceInst {
public:
	char* mInstID;
	void* mData;
	PSpiceDeviceModel* mModelData;
	void* mSignals;
	void* mStencil;
	void* mState;
};

class PSpiceDeviceMiscInfo {
public:
	double Temperature;
	double MeasurementTemperature;
	double Vt;
	double RelTol;
	unsigned int CurrentAnalysisNumber;
	double GMin;
	bool SkipBP; // Whether to skip bias point
	double CKTag[4]; // 0=> AG0 and so on
	double CKTomega; // ac frequency
};

class PSpiceSignalNodeList {
//data structure containing information about each of the nodes in the circuit.
public:
	char** mNodeNames;
	int mNodeCount;
};


/*! \section 1.2
 * Following API's need to be exposed by the Model Dll. Their pointers will be set by the Model Dll into PSpice Engine using the descSet* Functions described in section 1.3
 */

/**
 * Model Dll API called by PSpice 
 * To get any text printed in the PSpice out file
 * @param[in]	pDeviceInst	Device Instance Pointer
 * @return		String to be printed out
 */
typedef const char* (*pPrintDescription_t)(PSpiceDeviceInst* pDeviceInst);

/**
 * Model Dll API called by PSpice 
 * To install Models
 * @param	None
 * @return	None	
 */
typedef void  (*pInstallFunction_t)( void);

/**
 * Model Dll API called by PSpice 
 * To create and initialize a CMI Model
 * @param[in]	pDeviceModel	Device Model Pointer
 * @return		None
 */
typedef void (*pDefaultModel_t)( PSpiceDeviceModel* pDeviceModel);

/**
 * Model Dll API called by PSpice 
 * To Delete a CMI Model object
 * @param[in]	pDeviceModel	Device Model Pointer
 * @return		None
 */
typedef void (*pDeleteModel_t)( PSpiceDeviceModel* pDeviceModel);

/**
 * Model Dll API called by PSpice 
 * To set default model parameters
 * @param[in]	pDeviceModel	Device Model Pointer
 * @param[in]	ParamCount		Number of Model Parameters
 * @return		Status
 */
typedef int  (*pDefaultModelParams_t)( PSpiceDeviceModel* pDeviceModel, int ParamCount);

/**
 * Model Dll API called by PSpice 
 * To set specific model parameters
 * @param[in]	pDeviceModel	Device Instance Pointer
 * @param[in]	pParamVector	Vector of PSpiceCMIParam objects
 * @param[in]	pParamCount		Count of Parameters in pParamVector
 * @return		Status
 */
typedef int  (*pSetModelParams_t)( PSpiceDeviceModel* pDeviceModel, PSpiceCMIParam** pParamVector, int pParamCount);

/**
 * Model Dll API called by PSpice 
 * To Check validity of Model Parameters
 * @param[in]	pDeviceModel	Device Model Pointer
 * @return		String to be printed out
 */
typedef int  (*pModChk_t) (PSpiceDeviceModel* pDeviceModel);

/**
 * Model Dll API called by PSpice 
 * To Create and Initialize a Device Instance
 * @param[in]	pDeviceInst		Device Instance Pointer
 * @param[in]	pMiscInfo		Misc Simulation Info
 * @return		None
 */
typedef void (*pDefaultInstance_t)( PSpiceDeviceInst* pDeviceInst, PSpiceDeviceMiscInfo* pMiscInfo);

/**
 * Model Dll API called by PSpice 
 * To Delete an existing Device Instance
 * @param[in]	pDeviceInst		Device Instance Pointer
 * @return		None
 */
typedef void (*pDeleteInstance_t)( PSpiceDeviceInst* pDeviceInst);


/**
 * Model Dll API called by PSpice 
 * To Create and Initialize Signal (Port) Data for a Device Instance
 * @param[in]	pDeviceInst		Device Instance Pointer
 * @param[in]	pMiscInfo		Misc Simulation Info
 * @return		Status
 */
typedef int (*pDefaultSignals_t)( PSpiceDeviceInst* pDeviceInst, PSpiceDeviceMiscInfo* pMiscInfo);

/**
 * Model Dll API called by PSpice 
 * To Delete Signal Data for an existing Device Instance
 * @param[in]	pDeviceInst		Device Instance Pointer
 * @param[in]	pMiscInfo		Misc Simulation Info
 * @return		None
 */
typedef int (*pDeleteSignals_t)( PSpiceDeviceInst* pDeviceInst, PSpiceDeviceMiscInfo* pMiscInfo);

/**
 * Model Dll API called by PSpice 
 * To Create and Initialize Stencil (Matrix) Data for a Device Instance
 * @param[in]	pDeviceInst		Device Instance Pointer
 * @param[in]	pMiscInfo		Misc Simulation Info
 * @return		Status
 */
typedef int (*pDefaultStencil_t)( PSpiceDeviceInst* pDeviceInst, PSpiceDeviceMiscInfo* pMiscInfo);

/**
 * Model Dll API called by PSpice 
 * To Delete Stencil (Matrix) Data for an existing Device Instance
 * @param[in]	pDeviceInst		Device Instance Pointer
 * @param[in]	pMiscInfo		Misc Simulation Info
 * @return		None
 */
typedef int (*pDeleteStencil_t)( PSpiceDeviceInst* pDeviceInst, PSpiceDeviceMiscInfo* pMiscInfo);

/**
 * Model Dll API called by PSpice 
 * To Create and Initialize State Data for a Device Instance
 * @param[in]	pDeviceInst		Device Instance Pointer
 * @param[in]	pMiscInfo		Misc Simulation Info
 * @return		Status
 */
typedef int (*pDefaultState_t)( PSpiceDeviceInst* pDeviceInst, PSpiceDeviceMiscInfo* pMiscInfo);

/**
 * Model Dll API called by PSpice 
 * To Delete State Data for an existing Device Instance
 * @param[in]	pDeviceInst		Device Instance Pointer
 * @param[in]	pMiscInfo		Misc Simulation Info
 * @return		None
 */
typedef int (*pDeleteState_t)( PSpiceDeviceInst* pDeviceInst, PSpiceDeviceMiscInfo* pMiscInfo);

/**
 * Model Dll API called by PSpice 
 * To Set Device pin current for an instance
 * @param[in]	pDeviceInst		Device Instance Pointer
 * @param[in]	pPin			Pin Index
 * @param[in]	pMode			Simulation Mode
 * @return		Current value
 */
typedef double (*pSetDevicePinCurrent_t) (PSpiceDeviceInst* pDeviceInst, int pPin, int pMode);

/**
 * Model Dll API called by PSpice 
 * To set complex value of device pin current for an instance (called during ac analysis)
 * @param[in]	pDeviceInst		Device Instance Pointer
 * @param[in]	pPin			Pin Index
 * @param[in]	pOmega			Current Frequency
 * @param[out]	pCurrentReal	Real part of Current
 * @param[out]	pCurrentImag	Imaginary part of Current
 * @return		None
 */
typedef void   (*pSetDevicePinCurrentComplex_t) (PSpiceDeviceInst* pDeviceInst, int pin, double omega, double& pCurrentReal, double& pCurrentImag);

/**
 * Model Dll API called by PSpice 
 * To bind terminals to their signals
 * @param[in]	pDeviceInst		Device Instance Pointer
 * @param[in]	pMiscInfo		Misc Simulation Info
 * @param[in]	TerminalCount	Number of terminals
 * @param[in]	pNodeList		List of Device Terminals
 * @return		None
 */
typedef void   (*pBindTerminals_t)( PSpiceDeviceInst* pDeviceInst, PSpiceDeviceMiscInfo* pMiscInfo, int TerminalCount, PSpiceSignalNodeList* pNodeList);

/**
 * Model Dll API called by PSpice 
 * To reserve memory in matrix for all value items needed by the device
 * @param[in]	pDeviceInst		Device Instance Pointer
 * @param[in]	pMiscInfo		Misc Simulation Info
 * @param[in]	pNodeList		List of Device Terminals
 * @return		status
 */
typedef int    (*pReserveNodes_t)( PSpiceDeviceInst* pDeviceInst, PSpiceDeviceMiscInfo* pMiscInfo, PSpiceSignalNodeList* pNodeList); //, enum PSpiceError* pError );

/**
 * Model Dll API called by PSpice 
 * To get matrix pointers for all value items of a device
 * @param[in]	pDeviceInst		Device Instance Pointer
 * @return		None
 */
typedef void   (*pGetMatrixPointers_t)(PSpiceDeviceInst* InstKnot); 

/**
 * Model Dll API called by PSpice 
 * To add internal nodes for the device
 * @param[in]	pDeviceInst		Device Instance Pointer
 * @param[in]	pMiscInfo		Misc Simulation Info
 * @return		None
 */
typedef void   (*pAddInternalNodes_t) (PSpiceDeviceInst* InstKnot, PSpiceDeviceMiscInfo* pMiscInfo);

/**
 * Model Dll API called by PSpice 
 * To preLoad a device (called only once for a simulation)
 * @param[in]	pDeviceInst		Device Instance Pointer
 * @param[in]	pDeviceModel	Model Pointer
 * @return		status
 */
typedef int   (*pPreload_t)( PSpiceDeviceInst* pDeviceInst, PSpiceDeviceModel* pDeviceModel );

/**
 * Model Dll API called by PSpice 
 * To load ac (frequency-dependent) model of the device
 * @param[in]	pDeviceInst		Device Instance Pointer
 * @param[in]	Omega			AC Frequency
 * @return		Status
 */
typedef int   (*pAC_Load_t)( PSpiceDeviceInst* pDeviceInst, double Omega);

/**
 * Model Dll API called by PSpice 
 * To load transient model of the device
 * @param[in]	ModeFl		Mode flag
 * @param[in]	InitFl		Init flag
 * @param[in]	LoadFl		Load flag
 * @param[in]	Temperature		Simulation Temperature
 * @param[in]	pDeviceInst		Device Instance Pointer
 * @param[in]	pMiscInfo		Misc Simulation Info
 * @return		Device Convergence status
 */
typedef int   (*pTranLoad_t) (int ModeFl, int InitFl, int LoadFl, double pTemperature, PSpiceDeviceInst* pDeviceInst, PSpiceDeviceMiscInfo* pMiscInfo);

/**
 * Model Dll API called by PSpice 
 * For PWL Analysis. Evaluates distance of each model from the constant slope region boundary
 * enum value = PSPICE_GET_INTERCEPT
 * @param[in]	pDeviceInst		Device Instance Pointer
 * @return	Intercept value
 */
typedef double  (*pGetIntercept_t)(PSpiceDeviceInst* pDeviceInst);
	
/**
 * Model Dll API called by PSpice 
 * To calculate device noise for a particular frequency
 * @param[in]	pDeviceInst		Device Instance Pointer
 * @param[in]	pTemp			Temperature
 * @param[in]	pOmega			Frequency
 * @return		Device Noise value
 */
typedef double (*pNoise_t) (PSpiceDeviceInst* pDeviceInst, double pTemp, double pOmega, double* comps, int pCompsLength);

/**
 * Model Dll API called by PSpice 
 * To get simulation break point for a device
 * @param[in]	pDeviceInst		Device Instance Pointer
 * @return		breakpoint value
 */
typedef double (*pGetBreakPoint_t) (PSpiceDeviceInst* pDeviceInst);

/**
 * Model Dll API called by PSpice 
 * To load temperature changes for a device instance
 * @param[in]	pDeviceInst		Device Instance Pointer
 * @param[in]	ToldK			Old Temperature
 * @param[in]	TnewK			New Temperature
 * @param[in]	TnomK			Nominal Temperature
 * @return		None
 */
typedef void (*pTmpModDevice_t) (PSpiceDeviceInst* pDeviceInst, double ToldK, double TnewK, double TnomK, PSpiceDeviceMiscInfo* mMiscInfo);

/**
 * Model Dll API called by PSpice 
 * To load temperature changes for a device model
 * @param[in]	pDeviceModel	Device Model Pointer
 * @param[in]	ToldK			Old Temperature
 * @param[in]	TnewK			New Temperature
 * @param[in]	TnomK			Nominal Temperature
 * @return		None
 */
typedef void (*pTmpModModel_t) (PSpiceDeviceModel* pDeviceModel, double ToldK, double TnewK, double TnomK);

/**
 * Model Dll API called by PSpice 
 * To get integration error for a device
 * @param[in]	pDeviceInst		Device Instance Pointer
 * @return		Current maximum time step supported by device
 */
typedef double (*pTrunc_t) (PSpiceDeviceInst* pDeviceInst);

/**
 * Model Dll API called by PSpice 
 * To save checkpoint for a device
 * @param[in]	pDeviceInst		Device Instance Pointer
 * @param[in]	pVersion		checkpoint version
 * @param[out]	pStr			checkpoint data			
 * @param[out]	pSize			checkpoint data size
 * @return		None
 */
typedef void (*pSaveCheckpoint_t) (void* pDeviceInst, unsigned int pVersion, void* pStr, size_t pSize);

/**
 * Model Dll API called by PSpice 
 * To load checkpoint for a device
 * @param[in]	pDeviceInst		Device Instance Pointer
 * @param[in]	pVersion		checkpoint version
 * @param[in]	pStr			checkpoint data			
 * @param[in]	pSize			checkpoint data size
 * @return		None
 */
typedef void (*pLoadCheckpoint_t) (void* pDeviceInst, unsigned int pVersion, void* pStr, size_t pSize);

/**
 * Model Dll API called by PSpice 
 * To check topology for the device
 * @param[in]	pDeviceInst		Device Instance Pointer
 * @param[in]	pVersion		checkpoint version
 * @param[in]	pStr			topology string
 * @return		status
 */
typedef bool (*pCheckTopology_t) (void* pDeviceInst, unsigned int pVersion, void* pStr);

/**
 * Model Dll API called by PSpice 
 * To save topology for the device
 * @param[in]	pDeviceInst		Device Instance Pointer
 * @param[in]	pVersion		checkpoint version
 * @param[out]	pStr			topology string
 * @return		None
 */
typedef void (*pSaveTopology_t) (void* pDeviceInst, unsigned int pVersion, void* pStr);

/**
 * Model Dll API called by PSpice 
 * To set topology size
 * @param[in]	pDeviceInst		Device Instance Pointer
 * @param[in]	pVersion		checkpoint version
 * @return		topology size
 */
typedef size_t (*pSetTopologySize_t) (void* pDeviceInst, unsigned int pVersion);


/*! \section 1.3
 * The Model Dll will set function pointers into PSpice Engine using the descSet* Functions described in this section.
 * CMI API templates - API templates starting with descSet* are used to set cmi dll function pointers into the engine
 * Dll must export a function " void PSpiceInstallFunction();  "
 * Inside this function, function pointers for each model install function must be set 
 * All function pointers are set by this function
 */


/**
 * Used for printing Name of Model
 * May be used in future - this information provided by this API is not used right now
 * @param[in]	handle	Model Reference
 * @param[in]	Title	Model Description/Title		
 * @return	None
 */
typedef void (*descSetTitle_t)(PrimitivePtr handle, char *Title);

/**
 * May be used in future - this information provided by this API is not used right now
 * @param[in]	handle				Model Reference
 * @param[in]	MinTerminalCount	Minimum number of terminals required by the model
 * @return	None
 */
typedef void (*descSetMinTerminalCount_t)(PrimitivePtr handle, int MinTerminalCount);

/**
 * May be used in future - this information provided by this API is not used right now
 * @param[in]	handle				Model Reference
 * @param[in]	MaxTerminalCount	Maximum number of terminals required by the model
 * @return	None
 */
typedef void (*descSetMaxTerminalCount_t)(PrimitivePtr handle, int MaxTerminalCount);

/**
 * May be used for printing in future - this information provided by this API is not used right now
 * @param[in]	handle				Model Reference
 * @param[in]	TerminalNames		List of all Internal Terminal Names of the Model
 * @return	None
 */
typedef void (*descSetTerminalNames_t)(PrimitivePtr handle, char** TerminalNames);

/**
 * @param[in]	handle				Model Reference
 * @param[in]	TerminalNameCount	Minimum number of terminals required by the model
 * @return	None
 */
typedef void (*descSetTerminalNameCount_t)(PrimitivePtr handle, int TerminalNameCount);

/**
 * @param[in]	handle				Model Reference
 * @param[in]	ModelDataStructSize	Size of Model Data
 * @return	None
*/
typedef void (*descSetModelDataStructSize_t)(PrimitivePtr handle, unsigned int ModelDataStructSize);

/**
 * @param[in]	handle				Model Reference
 * @param[in]	InstDataStructSize	Size of Instance Data
 * @return	None
*/
typedef void (*descSetInstDataStructSize_t)(PrimitivePtr handle, unsigned int InstDataStructSize);

/**
 * @param[in]	handle		Model Reference
 * @param[in]	procedure	function pointer to set the model to its default values
 * @return	None
*/
typedef void (*descSetDefaultModel_t)(PrimitivePtr handle, pDefaultModel_t procedure);


/**
 * @param[in]	handle		Model Reference
 * @param[in]	procedure	function pointer to set model parameters
 * @return	None
*/
typedef void (*descSetSetModelParams_t)(PrimitivePtr handle, pSetModelParams_t procedure);

/**
 * @param[in]	handle		Model Reference
 * @param[in]	procedure	function pointer to check model data for accuracy
 * @return	None
*/
typedef void (*descSetCheckModel_t)(PrimitivePtr handle, pModChk_t procedure);

/**
 * @param[in]	handle		Model Reference
 * @param[in]	procedure	function pointer to reset device instance
 * @return	None
*/
typedef void (*descSetDefaultInstance_t)(PrimitivePtr handle, pDefaultInstance_t procedure);

/**
 * @param[in]	handle		Model Reference
 * @param[in]	procedure	function pointer to set instance parameters
 * @return	None
*/
typedef void (*descSetSetInstanceParams_t)(PrimitivePtr handle, pSetInstanceParams_t procedure);

/**
 * @param[in]	handle		Model Reference
 * @param[in]	procedure	function pointer to bind terminals to their signals
 * @return	None
*/
typedef void (*descSetBindTerminals_t)(PrimitivePtr handle, pBindTerminals_t procedure);

/**
 * @param[in]	handle		Model Reference
 * @param[in]	procedure	function pointer to create default state
 * @return	None
*/
typedef void (*descSetDefaultState_t)(PrimitivePtr handle, pDefaultState_t procedure);

/**
 * @param[in]	handle		Model Reference
 * @param[in]	procedure	function pointer to reserve memory for node pointers in matrix
 * @return	None
*/
typedef void (*descSetReserveNodes_t)(PrimitivePtr handle, pReserveNodes_t procedure);

/**
 * @param[in]	handle		Model Reference
 * @param[in]	procedure	function pointer to get matrix locations for all matrix entries
 * @return	None
*/
typedef void (*descSetGetMatrixPointers_t)(PrimitivePtr handle, pGetMatrixPointers_t procedure);

/**
 * @param[in]	handle		Model Reference
 * @param[in]	procedure	function pointer to create internal nodes for the model
 * @return	None
*/
typedef void (*descSetAddInternalNodes_t)(PrimitivePtr handle, pAddInternalNodes_t procedure);

/**
 * @param[in]	handle		Model Reference
 * @param[in]	procedure	function pointer to preload a device (called only once)
 * @return	None
*/
typedef void (*descSetPreload_t)(PrimitivePtr handle, pPreload_t procedure);

/**
 * @param[in]	handle		Model Reference
 * @param[in]	procedure	function pointer to run ac analysis on a device
 * @return	None
*/
typedef void (*descSetAC_Load_t)(PrimitivePtr handle, pAC_Load_t procedure);

/**
 * @param[in]	handle		Model Reference
 * @param[in]	procedure	function pointer to run transient analysis on a device
 * @return	None
*/
typedef void (*descSetTranLoad_t)(PrimitivePtr handle, pTranLoad_t procedure);

/**
 * @param[in]	handle		Model Reference
 * @param[in]	procedure	function pointer to get integration error for a device
 * @return	None
*/
typedef void (*descSetTrunc_t)(PrimitivePtr handle, pTrunc_t procedure);

/**
 * @param[in]	handle		Model Reference
 * @param[in]	procedure	function pointer for running noise analysis of a device
 * @return	None
 */
typedef void (*descSetNoise_t)(PrimitivePtr handle, pNoise_t procedure);

/**
 * @param[in]	handle		Model Reference
 * @param[in]	procedure	function pointer for running temperature analysis of a model
 * @return	None
 */
typedef void (*descSetTmpModModel_t)(PrimitivePtr handle, pTmpModModel_t procedure);

/**
 * @param[in]	handle		Model Reference
 * @param[in]	procedure	function pointer for running temperature analysis of a device
 * @return	None
 */
typedef void (*descSetTmpModDevice_t)(PrimitivePtr handle, pTmpModDevice_t procedure);

/**
 * @param[in]	handle					Model Reference
 * @param[in]	SignalDataStructSize	size of Signal Data
 * @return	None
 */
typedef void (*descSetSignalsStructSize_t)(PrimitivePtr handle, unsigned int SignalDataStructSize);

/**
 * @param[in]	handle					Model Reference
 * @param[in]	StencilDataStructSize	size of Stencil (Matrix) Data
 * @return	None
 */
typedef void (*descSetStencilStructSize_t)(PrimitivePtr handle, unsigned int StencilDataStructSize);

/**
 * @param[in]	handle				Model Reference
 * @param[in]	StateDataStructSize	size of State Data
 * @return	None
 */
typedef void (*descSetStateStructSize_t)(PrimitivePtr handle, unsigned int StateDataStructSize);

/**
 * @param[in]	handle			Model Reference
 * @param[in]	CheckpointSize	size of checkpoint data
 * @return	None
 */
typedef void (*descSetCheckPointSize_t)(PrimitivePtr handle, size_t CheckpointSize);

/**
 * @param[in]	handle		Model Reference
 * @param[in]	procedure	function pointer for saving checkpoint
 * @return	None
 */
typedef void (*descSetSaveCheckpoint_t)(PrimitivePtr handle, pSaveCheckpoint_t procedure);

/**
 * @param[in]	handle		Model Reference
 * @param[in]	procedure	function pointer for saving topology (used during checkpoint save)
 * @return	None
 */
typedef void (*descSetSaveTopology_t)(PrimitivePtr handle, pSaveTopology_t procedure);

/**
 * @param[in]	handle		Model Reference
 * @param[in]	procedure	function pointer for setting topology size (used during checkpoint save and restart)
 * @return	None
 */
typedef void (*descSetSetTopologySize_t)(PrimitivePtr handle, pSetTopologySize_t procedure);

/**
 * @param[in]	handle		Model Reference
 * @param[in]	procedure	function pointer for checking topology (Used during checkpoint restart)
 * @return	None
 */
typedef void (*descSetCheckTopology_t)(PrimitivePtr handle, pCheckTopology_t procedure);

/**
 * @param[in]	handle		Model Reference
 * @param[in]	procedure	function pointer for loading checkpoint
 * @return	None
 */
typedef void (*descSetLoadCheckpoint_t)(PrimitivePtr handle, pLoadCheckpoint_t procedure); 

/**
 * @param[in]	handle		Model Reference
 * @param[in]	procedure	function pointer for Setting Topology Size (Used during Checkpoint save and restart)
 * @return	None
 */
typedef void (*descSetSetTopologySize_t)(PrimitivePtr handle, pSetTopologySize_t procedure);

/**
 * @param[in]	handle		Model Reference
 * @param[in]	procedure	function pointer for Deleting Model object
 * @return	None
 */
typedef void (*descSetDeleteModel_t)(PrimitivePtr handle, pDeleteModel_t procedure);

/**
 * @param[in]	handle		Model Reference
 * @param[in]	procedure	function pointer for Deleting Instance object
 * @return	None
 */
typedef void (*descSetDeleteInstance_t)(PrimitivePtr handle, pDeleteInstance_t procedure);

/**
 * @param[in]	handle		Model Reference
 * @param[in]	procedure	function pointer for setting Current for each pin of a device
 * @return	None
 */
typedef void (*descSetSetDevicePinCurrent_t)(PrimitivePtr handle, pSetDevicePinCurrent_t procedure);

/**
 * @param[in]	handle		Model Reference
 * @param[in]	procedure	function pointer for setting complex current value for each pin of a device (for ac analysis)
 * @return	None
 */
typedef void (*descSetSetDevicePinCurrentComplex_t)(PrimitivePtr handle, pSetDevicePinCurrentComplex_t procedure);

/**
 * @param[in]	handle		Model Reference
 * @param[in]	procedure	function pointer for getting intercept value (for PWL Analysis)
 * @return	None
 */
typedef double(*descSetGetIntercept_t)(PrimitivePtr handle, pGetIntercept_t procedure);


//PWL
typedef bool (*pIsPWLModel_t)(void);
typedef const char* (*pPWLModelType_t)(void);

// returns length of double vectors
typedef int (*pGetPWLData_t)(void* pDeviceInst, double* pXVector, double* pYVector, int pSize);
typedef void (*pGetPWLDataStr_t)(void* pDeviceInst, const char* pValueStr, int pSize);

// mGetXLast
typedef double (*pGetLastVoltage_t)(void);

#endif
