/**
* Copyright by Cadence Design Systems 2014. All rights reserved
*/

#ifndef __PSpiceCommonAPIDefs_H__
#define __PSpiceCommonAPIDefs_H__

#include <list>
#include <vector>
#include <string>
#include <math.h>
#include <time.h>
#include <float.h>

class PSpiceState;
class PSpiceDelay;
class PSpiceConstraint;
class PSpiceInputSpec;
class PSpiceOutputSpec;

/**
 * File containing API's required by PSpice for loading external C++ model dll's for analog and digital models
 */

#ifdef  PSPICE
	#define PSP_CMI_EXPORT _declspec(dllexport)
#else
    #define PSP_CMI_EXPORT
#endif

#define CDLL_FUNC extern "C" __declspec(dllexport)


//! Mode Flags
/*! Mode controls some options during the iterations. */
enum modeFlags
{
	MDBPDC,     /*!< Small signal bias point calculation */
	MDBPTR,     /*!< Bias point for transient analysis */
	MDDCSW,     /*!< DC Sweep which uses previous step */
	MDTRAN,     /*!< Transient analysis */
	MDAC,        /*!< AC analysis */
	MDINITSMSIG,
	MDTRANOP,
	MDUIC,
	MDINITTRAN
};

//! Initialization Flags
/*! Init Flag controls where the device evaluation code gets its terminal voltages from. */
enum initFlags
{
	INNORM,     /*!< Use VltVct values from last iteration */
	ININIT,     /*!< Use IC= values in device table (transient bias point) or value calculated and stored in model table (reg. bias point) */
	INOFF,      /*!< Use VltVct unless device has OFF flag, then use 0 */
	INSTV0,     /*!< Use STVCT0 (= value used previous iteration) */
	INTRAN,     /*!< Use STVCT1 (= value used previous step) */
	INPRDCT     /*!< Extrapolate using STVCT2 and STVCT1 */
};

/**
 * Function used to set pointers to API's exposed from PSpice Engine
 * @param	List of function pointers to various functions - the order of pointers is given by enum PSpiceAPIs
 * @return	None
 */
CDLL_FUNC void __cdecl pspiceSetFunctionList(void** pPtr);

/**
 * PSpice searches for this function by name - this needs to be exported by the model dll
 * @param	None
 * @return	None
 */
CDLL_FUNC void __cdecl PSpiceInstallFunction();

CDLL_FUNC void __cdecl PSpiceInstallFunction1();

/**
 * PSpice Function templates - the templates starting with descSet* are used to set model dll function pointers into PSpice engine
 * Model Dll must export a function " void PSpiceInstallFunction();  "
 * Inside this function, function pointers for each model install function must be set 
 */

/**
* Function templates to be used by model dll to set and call functions in PSpice
*/
/**
 * Set function pointer to Model Installation Function
 * Function used to set pointer to model install function by model dll
 * enum value = PSPICE_SET_INSTALL_FUNC
 * Usage Example: 
 * pdescSetInstallFunction_t fp_descSetInstallFunction;
 * fp_descSetInstallFunction(&installCap);
 * @param	Model install function 
 * @return	None
 */
typedef void(*pFnPtr_t)(void*);
typedef void (*descSetInstallFunction_t)(pFnPtr_t);

typedef void(*pFnPtr1_t)(void*, void** pFunctionPointerList);
//typedef void (*descSetInstallFunction1_t)(pFnPtr1_t, void** pFnPtrList);

/**
 * Another type of install function - it returns the function pointers directly in a pre-defined order
 * This will be used for PWL Models for now
 * Expected return data - 
 * bool isPWLModel(void)
 * @return	Fixed-size List of function pointers in a pre-defined order
 */
typedef void (*descSetInstallFunction1_t)(pFnPtr1_t);



/**
 * @param	handle		Model Reference (created by PSpice Engine - passed on to Model dll)
 * @param	CMIVersion	CMI Version (Supported value = 1.0)
 * @return	None
 */
typedef void (*descSetVersion_t)(void* handle, const char* pVersion);

/**
 * @param	handle	Model Reference
 * @param	Name	Model Name
 * @return	None
 */
typedef void (*descSetName_t)(void* handle, const char *Name);

/**
 * Following functions allow the model dll to set specific function pointers for each model.
 * Function Templates Exposed by PSpice Engine dll
 */

typedef const char* (*pPSpiceGetLicenseString_t)();
typedef const char* (*pPSPICEGetOptionsParams_t)();

/**
 * Set Simulation Temperature
 * enum value = PSPICE_SET_SIMTEMP
 * @param	Temperature value (in Celsius)
 * @return	None
 */
typedef void (*pPSpiceSetSimulationTemperature_t)(double pTemperature);

/**
 * Set Title of Probe Section
 */
typedef void (*pPSpiceSetProbeTitle_t)(const char* pTitle);

/**
 * @return	Current Analog TIME value
 */
typedef double (*pPSpiceGetCurrentAnalogTime_t)(void);

/**
 * @return	Current Digital TIME value
 */
typedef double (*pPSpiceGetCurrentDigitalTime_t)(void);

/**
 * @return	Pointer to device
 */
typedef void* (*pPSpiceGetDevice_t)(const char*);

/**
 * Get value of global parameter from PSpice
 * enum value = PSPICE_GET_PARAMVALUE
 * @return	Value as string
 */
typedef const char* (*pPSpiceGetParamValue_t)(const char* );

/**
 * Get value of global parameter from PSpice
 * enum value = PSPICE_GET_PARAMVALUEDBL
 * @return	double value of parameter if it exists
 *			else MAXREAL
 */
typedef double (*pPSpiceGetParamValueDbl_t) (const char*);

typedef void (*pPSpiceWriteToOut_t)(const char* pText);

class PSpiceCMIParam;
typedef int (*pDefaultInstanceParams_t)( void* pInstKnot, int ParamCount);
typedef int (*pSetInstanceParams_t)( void* pInstKnot,  PSpiceCMIParam**, int pParamCount );

//! PSpice APIs Enumeration
/*! These are used to set the function pointers into the Model dll.
 *\nThese are used by pspiceSetFunctionList
 */
enum PSpiceAPIs {
	PSPICE_SET_INSTALLFUNC = 1, /*!< Both A&D: Install CMI Model */
	PSPICE_SET_SIMTEMP, /*!< Both A&D: Set simulation temperature */
	PSPICE_SET_PROBETITLE, /*!< Both A&D: Set probe section title */
	PSPICE_GET_ANALOGTIME, /*!< Analog: Get Time */
	PSPICE_GET_DIGITALTIME, /*!< Digital: Get Time */
	PSPICE_GET_PARAMVALUE, /*!< Both A&D: Get parameter value as string */
	PSPICE_GET_PARAMVALUEDBL, /*!< Both A&D: Get parameter value as double */
	PSPICE_SET_DELAY, /*!< Digital: Set Delay */
	PSPICE_SET_CONSTRAINT, /*!< Digital: Set constraint */
	PSPICE_SET_INSPEC, /*!< Digital: Setup Input Buffer specification */
	PSPICE_SET_OUTSPEC, /*!< Digital: Setup Output Buffer specification */
	PSPICE_GET_INSPEC, /*!< Digital: Get Input Buffer specification */
	PSPICE_GET_OUTSPEC, /*!< Digital: Get Output Buffer specification */
	PSPICE_GET_TIMINGVALUE, /*!< Digital: get Timing specification */
	PSPICE_ADD_INTERNALNODE, /*!< Analog: Add internal node */
	PSPICE_GET_MATRIXPTR, /*!< Analog: Get Matrix pointer for a value item */
	PSPICE_APPLY_VALUEITEM, /*!< Analog: Set value to a matrix pointer */
	PSPICE_APPLY_VALUEITEMCMPLX, /*!< Analog: Set value to a complex matrix pointer */
	PSPICE_GET_V, /*!< Analog: Get voltage for a node from simulator */
	PSPICE_GET_VI, /*!< Analog */
	PSPICE_GET_DELTA, /*!< Analog: Get current time step */
	PSPICE_GET_DELTAPREV, /*!< Analog: Get previous time steps */
	PSPICE_VOLTAGE_TOL, /*!< Analog: Get voltage tolerance */
	PSPICE_CURRENT_TOL, /*!< Analog: Get current tolerance */
	PSPICE_INTEGRATE, /*!< Analog: integrate */
	PSPICE_GET_STATE, /*!< Analog: Get current State Index*/
	PSPICE_GMINLOAD, /*!< Analog: Load for Gmin Stepping */
	PSPICE_GET_ACFREQUENCY, /*!< Analog: get frequency for ac analysis */
	PSPICE_SET_INSTDATASTRUCTSIZE, /*!< Analog: Set inst data size (descSet) */
	PSPICE_SET_MODELDATASTRUCTSIZE, /*!< Analog: set Model data size (descSet) */
	PSPICE_SET_SIGNALSTRUCTSIZE, /*!< Analog: set signal data size (descSet) */
	PSPICE_SET_STATESTRUCTSIZE, /*!< Analog: set state data size (descSet) */
	PSPICE_SET_RESERVENODES, /*!< Analog: Reserve memory in matrix (descSet) */
	PSPICE_SET_ADDINTERNALNODES, /*!< Analog: Add internal nodes for model (descSet) */
	PSPICE_SET_GETMATRIXPTR, /*!< Analog: Get matrix pointers for all value items (descSet) */
	PSPICE_SET_BINDTERMINALS, /*!< Analog: Bind terminals to matrix locations (descSet) */
	PSPICE_SET_DEFAULTSTATE, /*!< Unused */
	PSPICE_SET_MODELPARAMDESC, /*!< Analog: Set Model parameters (descSet) */
	PSPICE_SET_STENCILSTRUCTSIZE, /*!< Analog: Set stencil (matrix) size (descSet) */
	PSPICE_SET_PRELOAD, /*!< Analog: Set Preload for device (descSet) */
	PSPICE_SET_TRANLOAD, /*!< Analog: Set Transient load for device (descSet) */
	PSPICE_SET_ACLOAD, /*!< Analog: Set AC load for device (descSet) */
	PSPICE_SET_TRUNC, /*!< Analog: Get integration error (descSet) */
	PSPICE_SET_NOISE, /*!< Analog: Set noise load for device (descSet) */
	PSPICE_SET_GETBREAKPOINT, /*!< Analog: Get breakpoint (descSet) */
	PSPICE_SET_TMPMODMODEL, /*!< Analog: Set Temperature load for model (descSet) */
	PSPICE_SET_TMPMODDEVICE, /*!< Analog: Set Temperature load for device (descSet) */
	PSPICE_SET_DEFMODEL, /*!< Analog: Create Default model (descSet) */
	PSPICE_SET_MODELPARAMS, /*!< Analog: Set Model Parameters (descSet) */
	PSPICE_SET_MODCHK, /*!< Analog: Check Model (descSet) */
	PSPICE_SET_DEFINST, /*!< Analog: Create default device (descSet) */
	PSPICE_SET_INSTPARAMS, /*!< Analog: Set device parameters (descSet) */
	PSPICE_SET_VERSION, /*!< Both A&D: Set CMI Version (descSet) */
	PSPICE_SET_NAME, /*!< Both A&D: Set Name (descSet) */
	PSPICE_UPDATE_SV, /*!< Analog: Update state vector */
	PSPICE_CURRENT_TERR, /*!< Analog: Get current integration error */
	PSPICE_SET_PRINTDESC, /*!< Both A&D: Print Description (descSet) */
	PSPICE_SET_SAVECHECKPT, /*!< Both A&D: Save checkpoint (descSet) */
	PSPICE_SET_LOADCHECKPT, /*!< Both A&D: Load checkpoint (descSet) */
	PSPICE_SET_SAVETOPOLOGY, /*!< Both A&D: Save topology (for checkpoint) (descSet) */
	PSPICE_SET_CHECKTOPOLOGY, /*!< Both A&D: Check topology (for checkpoint) (descSet) */
	PSPICE_SET_DELETEINST, /*!< Analog: Delete device  (descSet) */
	PSPICE_SET_DELETEMODEL, /*!< Analog: Delete Model (descSet) */
	PSPICE_SET_SETPINI, /*!< Analog: Set current value (descSet) */
	PSPICE_SET_SETPINICMPLX, /*!< Analog: Set complex current value (for ac analysis) (descSet) */
	PSPICE_DIG_CREATEDEVICE, /*!< Digital: Create device */
	PSPICE_DIG_DELETEDEVICE, /*!< Digital: Delete Device */
	PSPICE_DIG_GETTERMCOUNT, /*!< Digital: Get Terminal count */
	PSPICE_DIG_SETTERMCOUNT, /*!< Digital: Set terminal count */
	PSPICE_DIG_EVALDEVICE, /*!< Digital: Evaluate device */
	PSPICE_DIG_SETTERM, /*!< Digital: Set Terminal value */
	PSPICE_DIG_GETTERM, /*!< Digital: Get Terminal value */
	PSPICE_DIG_GETTERMTYPE, /*!< Digital: Get Terminal Type */
	PSPICE_DIG_SETPARAM, /*!< Digital: Unused */
	PSPICE_DIG_SETPRINTDESC, /*!< Digital: Print Description of device to out file*/
	PSPICE_SET_WRITETOOUT, /*!< Both A&D: Write text to out file */
	PSPICE_DIG_FNCHANGED, /*!< Digital: Get State change */
	PSPICE_DIG_FNTRANSITION, /*!< Digital: get transition */
	PSPICE_DIG_INITDEV, /*!< Digital: Initialize Device */
	PSPICE_DIG_SETSTATE, /*!< Digital: set state */
	PSPICE_DIG_GETPARAMVALUE, /*!< Both A&D: Get parameter value as string */
	PSPICE_DIG_TICK_FROM_TIME, /*!< Digital: get digital ticks value */
	PSPICE_DIG_TIME_FROM_TICK, /*!< Digital: get digital time value */
	PSPICE_SET_GETTOPOLOGYSIZE, /*!< Both A&D: get topology size for checkpoint */
	PSPICE_SET_CHECKPTSIZE, /*!< Both A&D: set checkpoint size */
	PSPICE_GET_LICENSE, /*!< Both A&D: get name of license being used by PSpice */
	PSPICE_GET_OPTIONPARAMS, /*!< Both A&D: get option value */
	PSPICE_ADJUST_VALUEITEM, /*!< Analog: Add to matrix value item */
	PSPICE_GET_RHSPTR, /*!< Analog: Get RHS pointer for the matrix */
	PSPICE_ADD_INTERNALNODEBYNAME, /*!< Analog: Add internal node by name*/
	PSPICE_INTERNAL0, /*!< For internal use only */
	PSPICE_GET_RHSPTR1, /*!< For PWL Simulation */
	PSPICE_SET_GETINTERCEPT, /*!< For PWL Simulation - get intercept */
	PSPICE_SET_INSTALLFUNC1, /*!< New version of install - takes a list of function pointers as parameter */
	PSPICE_SET_PWLDATA_DBL, /*!< Set PWL Data as vectors of x and y values */
	PSPICE_SET_PWLDATA_STR, /*!< Set PWL Data as a single string value */
};

//! PSpice Parameter Value Types
enum PSpiceValueType {
	PSPICE_VALUE_REALTYPE, /*!< Double value */
	PSPICE_VALUE_STRINGTYPE, /*!< String value */
	PSPICE_VALUE_EXPRTYPE /*!< Expression - evaluated to a double value by PSpice engine */
};

//! PSpice Parameter Descriptor
class PSpiceParamDesc {
public:
	char Name[1024]; /*!< Parameter Name */
	short mVersion; /*!< Parameter Version (currently unused) */
};

//! PSpice Parameter Value subclass
class PSpiceAnyScalar {
public:
	double Real; /*!< Double value */ 
	std::string String; /*!< String value */
	void* exp;  /*!< Expression value */
};

//! PSpice Parameter Value Class
class PSpiceAnyValue {
public:
	PSpiceValueType Type; /*!< Value Type */
	PSpiceAnyScalar Scalar; /*!< Actual Value */
};

//! PSpice Parameter Top-level class
class PSpiceCMIParam {
public:
	PSpiceParamDesc *Desc; /*!< PSpice Parameter Descriptor */
	PSpiceAnyValue Value;  /*!< PSpice Parameter Value */

	PSpiceCMIParam() {
		Desc = new PSpiceParamDesc;
		Desc->mVersion = 1;
	}
	~PSpiceCMIParam() {
		delete Desc;
		Desc = NULL;
	}
};



#endif
